@@ -1,5 +1,5 @@
-#ifndef AUTO_CONFIG_BIT_VECTORS
-#define AUTO_CONFIG_BIT_VECTORS
+#ifndef MODULE_BIT_VECTOR
+#define MODULE_BIT_VECTOR
 /*****************************************************************************/
 /*  MODULE NAME:  BitVector.h                           MODULE TYPE:  (adt)  */
 /*****************************************************************************/
@@ -9,17 +9,37 @@
 #include <limits.h>                                 /*  MODULE TYPE:  (sys)  */
 #include <string.h>                                 /*  MODULE TYPE:  (sys)  */
 #include <ctype.h>                                  /*  MODULE TYPE:  (sys)  */
-#include "Definitions.h"                            /*  MODULE TYPE:  (dat)  */
+#include "ToolBox.h"                                /*  MODULE TYPE:  (dat)  */
 /*****************************************************************************/
 /*  MODULE INTERFACE:                                                        */
 /*****************************************************************************/
 
-/* ===> INTERNAL FUNCTIONS: <=== */
+typedef enum
+    {
+        ErrCode_Ok = 0,   /* everything went allright                        */
 
-N_word  BitVector_Auto_Config(void);                 /* 0 = ok, 1..7 = error */
+        ErrCode_Type,     /* types word and size_t have incompatible sizes   */
+        ErrCode_Bits,     /* bits of word and sizeof(word) are inconsistent  */
+        ErrCode_Word,     /* size of word is less than 16 bits               */
+        ErrCode_Long,     /* size of word is greater than size of long       */
+        ErrCode_Powr,     /* number of bits of word is not a power of two    */
+        ErrCode_Loga,     /* error in calculation of logarithm               */
+        ErrCode_Null,     /* unable to allocate memory for bitmask table     */
 
-N_word  BitVector_Size  (N_int elements);   /* bit vector size (# of words)  */
-N_word  BitVector_Mask  (N_int elements);   /* bit vector mask (unused bits) */
+        ErrCode_Size,     /* bit vector size mismatch error                  */
+        ErrCode_Same,     /* operands must (all) be distinct bit vectors     */
+        ErrCode_Zero,     /* division by zero attempted                      */
+        ErrCode_Crea,     /* unable to create temporary bit vector(s)        */
+        ErrCode_Ovfl,     /* numerical overflow error                        */
+        ErrCode_Form      /* format error in decimal number input string     */
+    } ErrCode;
+
+/* ===> MISCELLANEOUS: <=== */
+
+ErrCode BitVector_Boot       (void);                 /* 0 = ok, 1..7 = error */
+
+N_word  BitVector_Size  (N_int bits);       /* bit vector size (# of words)  */
+N_word  BitVector_Mask  (N_int bits);       /* bit vector mask (unused bits) */
 
 /* ===> CLASS METHODS: <=== */
 
@@ -28,14 +48,16 @@ charptr BitVector_Version    (void);               /* returns version string */
 N_int   BitVector_Word_Bits  (void);    /* returns # of bits in machine word */
 N_int   BitVector_Long_Bits  (void);   /* returns # of bits in unsigned long */
 
-wordptr BitVector_Create(N_int elements, boolean clear);          /* malloc  */
+wordptr BitVector_Create(N_int bits, boolean clear);              /* malloc  */
 
 /* ===> OBJECT METHODS: <=== */
 
-wordptr BitVector_Shadow  (wordptr addr); /* create new, same size but empty */
-wordptr BitVector_Clone   (wordptr addr);          /* create exact duplicate */
+wordptr BitVector_Shadow  (wordptr addr);  /* makes new, same size but empty */
+wordptr BitVector_Clone   (wordptr addr);           /* makes exact duplicate */
 
-wordptr BitVector_Resize  (wordptr oldaddr, N_int elements);      /* realloc */
+wordptr BitVector_Concat  (wordptr X, wordptr Y);   /* returns concatenation */
+
+wordptr BitVector_Resize  (wordptr oldaddr, N_int bits);          /* realloc */
 void    BitVector_Destroy (wordptr addr);                         /* free    */
 
 /* ===> bit vector copy function: */
@@ -50,11 +72,16 @@ void    BitVector_Flip    (wordptr addr);                   /* X = ~X        */
 
 void    BitVector_Primes  (wordptr addr);
 
+/* ===> miscellaneous functions: */
+
+void    BitVector_Reverse (wordptr X, wordptr Y);
+
 /* ===> bit vector interval operations and functions: */
 
 void    BitVector_Interval_Empty   (wordptr addr, N_int lower, N_int upper);
 void    BitVector_Interval_Fill    (wordptr addr, N_int lower, N_int upper);
 void    BitVector_Interval_Flip    (wordptr addr, N_int lower, N_int upper);
+void    BitVector_Interval_Reverse (wordptr addr, N_int lower, N_int upper);
 
 boolean BitVector_interval_scan_inc(wordptr addr, N_int start,
                                     N_intptr min, N_intptr max);
@@ -70,11 +97,12 @@ wordptr BitVector_Interval_Substitute(wordptr X, wordptr Y,
 
 /* ===> bit vector test functions: */
 
-boolean BitVector_is_empty(wordptr addr);                   /* X == {} ?     */
-boolean BitVector_is_full (wordptr addr);                   /* X == ~{} ?    */
+boolean BitVector_is_empty         (wordptr addr);          /* X == {} ?     */
+boolean BitVector_is_full          (wordptr addr);          /* X == ~{} ?    */
 
-boolean BitVector_equal   (wordptr X, wordptr Y);           /* X == Y ?      */
-Z_int   BitVector_Compare (wordptr X, wordptr Y);           /* X <,=,> Y ?   */
+boolean BitVector_equal            (wordptr X, wordptr Y);  /* X == Y ?      */
+Z_int   BitVector_Lexicompare      (wordptr X, wordptr Y);  /* X <,=,> Y ?   */
+Z_int   BitVector_Compare          (wordptr X, wordptr Y);  /* X <,=,> Y ?   */
 
 /* ===> bit vector string conversion functions: */
 
@@ -85,7 +113,7 @@ charptr BitVector_to_Bin  (wordptr addr);
 boolean BitVector_from_bin(wordptr addr, charptr string);
 
 charptr BitVector_to_Dec  (wordptr addr);
-boolean BitVector_from_dec(wordptr addr, charptr string);
+ErrCode BitVector_from_dec(wordptr addr, charptr string);
 
 charptr BitVector_to_Enum (wordptr addr);
 boolean BitVector_from_enum(wordptr addr, charptr string);
@@ -115,8 +143,10 @@ void    BitVector_Move_Right  (wordptr addr, N_int bits);
 
 /* ===> bit vector insert/delete bits: */
 
-void    BitVector_Insert      (wordptr addr, N_int offset, N_int count);
-void    BitVector_Delete      (wordptr addr, N_int offset, N_int count);
+void    BitVector_Insert      (wordptr addr, N_int offset, N_int count,
+                               boolean clear);
+void    BitVector_Delete      (wordptr addr, N_int offset, N_int count,
+                               boolean clear);
 
 /* ===> bit vector arithmetic: */
 
@@ -128,11 +158,11 @@ boolean BitVector_subtract(wordptr X, wordptr Y, wordptr Z, boolean carry);
 void    BitVector_Negate  (wordptr X, wordptr Y);
 void    BitVector_Absolute(wordptr X, wordptr Y);
 Z_int   BitVector_Sign    (wordptr addr);
-boolean BitVector_Mul_Pos (wordptr X, wordptr Y, wordptr Z);
-N_int   BitVector_Multiply(wordptr X, wordptr Y, wordptr Z);
-void    BitVector_Div_Pos (wordptr Q, wordptr X, wordptr Y, wordptr R);
-boolean BitVector_Divide  (wordptr Q, wordptr X, wordptr Y, wordptr R);
-boolean BitVector_GCD     (wordptr X, wordptr Y, wordptr Z);
+ErrCode BitVector_Mul_Pos (wordptr X, wordptr Y, wordptr Z);
+ErrCode BitVector_Multiply(wordptr X, wordptr Y, wordptr Z);
+ErrCode BitVector_Div_Pos (wordptr Q, wordptr X, wordptr Y, wordptr R);
+ErrCode BitVector_Divide  (wordptr Q, wordptr X, wordptr Y, wordptr R);
+ErrCode BitVector_GCD     (wordptr X, wordptr Y, wordptr Z);
 
 /* ===> direct memory access functions: */
 
@@ -151,10 +181,10 @@ void    BitVector_Word_Delete (wordptr addr, N_int offset, N_int count,
 
 /* ===> arbitrary size chunk functions: */
 
-void    BitVector_Chunk_Store (wordptr addr, N_int offset,
-                               N_int chunksize, N_long value);
-N_long  BitVector_Chunk_Read  (wordptr addr, N_int offset,
-                               N_int chunksize);
+void    BitVector_Chunk_Store (wordptr addr, N_int chunksize,
+                               N_int offset, N_long value);
+N_long  BitVector_Chunk_Read  (wordptr addr, N_int chunksize,
+                               N_int offset);
 
 /* ===> set operations: */
 
@@ -183,92 +213,6 @@ void    Matrix_Closure       (wordptr addr, N_int rows, N_int cols);
 void    Matrix_Transpose     (wordptr X, N_int rowsX, N_int colsX,
                               wordptr Y, N_int rowsY, N_int colsY);
 
-/*
-// The "mask" that is used in various functions throughout this package avoids
-// problems that may arise when the number of bits used in a bit vector (or the
-// number of elements used in a set) is not a multiple of 16 (or whatever the
-// size of a machine word is on your system).
-//
-// (Note that in this package, the type name "N_word" is used instead of "word"
-// in order to avoid possible name conflicts with any system header files on
-// some machines!)
-//
-// In these cases, comparisons between bit vectors (or sets) would fail to
-// produce the expected results if in one vector (or set) the unused bits
-// were set, while they were cleared in the other. To prevent this, unused
-// bits are systematically turned off by this package using this "mask".
-//
-// If the number of elements in a set is, say, 500, you need to define a
-// contiguous block of memory with 32 words (if a machine word (= a "N_word")
-// is worth 16 bits) to store any such set, or
-//
-//                          N_word your_set[32];
-//
-// 32 words contain a total of 512 bits, which means (since only one bit
-// is needed for each element to flag its presence or absence in the set)
-// that 12 bits remain unused.
-//
-// Since element #0 corresponds to bit #0 in word #0 of "your_set", and
-// element 499 corresponds to bit #3 in word #31, the 12 most significant
-// bits of word #31 remain unused.
-//
-// Therefore, the mask word should have the 12 most significant bits cleared
-// while the remaining lower bits remain set; in the case of our example,
-// the mask word would have the value 0x000F.
-//
-// Sets or bit vectors in this package cannot be defined like variables in C,
-// however.
-//
-// In order to use a set variable or bit vector, you have to invoke the
-// "object constructor method" "BitVector_Create()", which dynamically creates
-// a set variable or bit vector of the desired size (maximum number of ele-
-// ments) by allocating the appropriate amount of memory on the system heap
-// and initializing it to represent an empty set (all bits cleared).
-//
-// MNEMONIC: If the name of one of the functions in this package (that is,
-//           the part of the name after the prefix "BitVector_", "Set_" etc.)
-//           consists of only lower case letters, this indicates that it
-//           returns a boolean function result.
-//
-// REMINDER: All indices into your set or bit vector range from zero to the
-//           maximum number of elements in your set (or total number of bits
-//           in your bit vector) minus one!
-//
-// WARNING:  It is your, the user's responsibility to make sure that all
-//           indices are within the appropriate range for any given set or
-//           bit vector! No bounds checking is performed by the functions of
-//           this package! If you don't, you may get core dumps and receive
-//           "segmentation violation" errors. To do bounds checking in your
-//           application, check incoming indices as follows:
-//           An index is invalid for any set or bit vector (given by its
-//           pointer "addr") if it is greater than or equal to "bits_(addr)"
-//           (or if it is less than zero). Note that you don't need to check
-//           for negative indices, though, because the type used for indices
-//           in this package is UNSIGNED.
-//
-// WARNING:  You shouldn't perform any set operations with sets of different
-//           sizes unless you know exactly what you're doing. If the resulting
-//           set is larger than the argument sets, or if the argument sets are
-//           of different sizes, this may result in a segmentation violation
-//           error, because you are actually reading beyond the allocated
-//           length of the argument sets. If the resulting set is smaller
-//           than the two argument sets, and if the two argument sets have
-//           the same size, no error occurs, but you will of course lose
-//           some information (the result will be truncated to the size
-//           of the resulting set).
-//
-// NOTE:     The auto-config initialization routine can fail for 7 reasons:
-//
-//           Result:                    Meaning:
-//
-//              1      The type "N_int" is larger (has more bits) than "size_t"
-//              2      The number of bits in a word and sizeof(N_int)*8 differ
-//              3      The number of bits of a machine word is less than 16
-//              4      The type "N_int" is larger (has more bits) than "N_long"
-//              5      The number of bits of a machine word is not a power of 2
-//              6      Internal error: "bits" and "2^ld(bits)" differ
-//              7      The attempt to allocate memory with "malloc()" failed
-*/
 /*****************************************************************************/
 /*  MODULE RESOURCES:                                                        */
 /*****************************************************************************/
@@ -300,8 +244,19 @@ void    Matrix_Transpose     (wordptr X, N_int rowsX, N_int colsX,
 /*    14.04.97    Version 4.0                                                */
 /*    30.06.97    Version 4.1  added word-ins/del, move-left/right, inc/dec  */
 /*    16.07.97    Version 4.2  added is_empty, is_full                       */
-/*    12.10.97    Version 5.0                                                */
+/*    17.02.98    Version 5.0                                                */
 /*****************************************************************************/
-/*  COPYRIGHT (C) 1989-1997 BY:  Steffen Beyer         ALL RIGHTS RESERVED.  */
+/*  COPYRIGHT:                                                               */
+/*                                                                           */
+/*    Copyright (c) 1995, 1996, 1997, 1998 by Steffen Beyer.                 */
+/*    All rights reserved.                                                   */
+/*                                                                           */
+/*    This piece of software is "Non-Profit-Ware" ("NP-ware").               */
+/*                                                                           */
+/*    You may use, copy, modify and redistribute it under the terms of the   */
+/*    "Non-Profit License" (NPL).                                            */
+/*                                                                           */
+/*    Please refer to the file "LICENSE" in this distribution for details!   */
+/*                                                                           */
 /*****************************************************************************/
 #endif
