@@ -1,6 +1,6 @@
 ##
 ##  slice_pass1.pl -- Pass 1
-##  Copyright (c) 1997 Ralf S. Engelschall, All Rights Reserved. 
+##  Copyright (c) 1997,1998 Ralf S. Engelschall. 
 ##
 
 package main;
@@ -116,20 +116,19 @@ sub pass1 {
 
     #   check: were all opened slices really closed?
     if ($CURRENT_LEVEL_SET->Norm > 0) {
-        print STDERR "ERROR: Some slices were not closed: ";
         my $i;
+        my $err = '';
         for ($i = 0; $i <= $CURRENT_LEVEL_SET->Max(); $i++) {
             if ($CURRENT_LEVEL_SET->bit_test($i)) {
                 my $name;
                 foreach $name (keys(%CURRENT_LEVEL_BRAIN)) {
                     if ($CURRENT_LEVEL_BRAIN{$name} == ($i+1)) {
-                        print STDERR "`$name' ";
+                        $err .= " `$name'";
                     }
                 }
             }
         }
-        print STDERR "\n";
-        exit(1);
+        &error("Some slices were not closed:$err\n");
     }
 }
 
