@@ -1,6 +1,6 @@
 ##
 ##  slice_util.pl -- Utility functions
-##  Copyright (c) 1997 Ralf S. Engelschall, All Rights Reserved. 
+##  Copyright (c) 1997,1998 Ralf S. Engelschall. 
 ##
 
 package main;
@@ -9,16 +9,22 @@ sub verbose {
     my ($str) = @_;
 
     if ($main::CFG->{OPT}->{X}) {
-        $str =~ s|^|** SLICE:Verbose: |mg;
+        $str =~ s|^|** Slice:Verbose: |mg;
         print STDERR $str;
     }
 }
 
+sub printerror {
+    my ($str) = @_;
+
+    $str =~ s|^|** Slice:Error: |mg;
+    print STDERR $str;
+}
+
 sub error {
     my ($str) = @_;
 
-    $str =~ s|^|** SLICE:Error: |mg;
-    print STDERR $str;
+    &printerror($str);
     exit(1);
 }
 
